#include "../StdAfx.h"
#include "TsStream.h"

#define CODE_JPN_KANJI            0
#define CODE_JPN_ALPHANUMERIC     1
#define CODE_JPN_HIRAGANA         2
#define CODE_JPN_KATAKANA         3
#define CODE_JPN_MOSAIC_A         4
#define CODE_JPN_MOSAIC_B         5
#define CODE_JPN_MOSAIC_C         6
#define CODE_JPN_MOSAIC_D         7
#define CODE_JPN_PRO_ALPHANUMERIC 8
#define CODE_JPN_PRO_HIRAGANA     9
#define CODE_JPN_PRO_KATAKANA     10
#define CODE_JPN_X0201_KATAKANA   11
#define CODE_JPN_JIS_KANJI_1      12
#define CODE_JPN_JIS_KANJI_2      13
#define CODE_JPN_ADDITIONAL       14

#define CODE_JPN_MODE_NONE 0
#define CODE_JPN_MODE_GL   1
#define CODE_JPN_MODE_GR   2

static char *jpn_argb_alp_num_table = 
/*      0 1 2 3 4 5 6 7 8 9 A B C D E F */
/*0x*/"@@@@@@@@@@@@@@@@" 
/*1x*/"@@@@@@@@@@@@@@@@" 
/*2x*/"@Ihfij{C|D^" 
/*3x*/"OPQRSTUVWXFGH" 
/*4x*/"`abcdefghijklmn" 
/*5x*/"opqrstuvwxymnOQ" 
/*5x*/"@" 
/*6x*/"obpP@" ;

static char *jpn_argb_hiragana_table = 
/*      0 1 2 3 4 5 6 7 8 9 A B C D E F */
/*0x*/"@@@@@@@@@@@@@@@@" 
/*1x*/"@@@@@@@@@@@@@@@@" 
/*2x*/"@" 
/*3x*/"" 
/*4x*/"ÂĂłƂǂȂɂʂ˂̂" 
/*5x*/"΂ςЂт҂ӂԂՂւׂ؂قڂۂ܂" 
/*6x*/"ނ߂"
/*7x*/"@@@TU[BuvAE@" ;

static char *jpn_argb_katakana_table = 
/*      0 1 2 3 4 5 6 7 8 9 A B C D E F */
/*0x*/"@@@@@@@@@@@@@@@@" 
/*1x*/"@@@@@@@@@@@@@@@@" 
/*2x*/"@@ABCDEFGHIJKLMN" 
/*3x*/"OPQRSTUVWXYZ[\]^" 
/*4x*/"_`abcdefghijklmn" 
/*5x*/"opqrstuvwxyz{|}~" 
/*6x*/""
/*7x*/"RS[BuvAE@" ;

static char *jpn_argb_x0201_katakana_table = 
/*      0 1 2 3 4 5 6 7 8 9 A B C D E F */
/*0x*/"@@@@@@@@@@@@@@@@" 
/*1x*/"@@@@@@@@@@@@@@@@" 
/*2x*/"@BuvAE@BDFHb"
/*3x*/"[ACEGIJLNPRTVXZ\" 
/*4x*/"^`cegijklmnqtwz}" 
/*5x*/"~JK" 
/*6x*/"@@@@@@@@@@@@@@@@"
/*7x*/"@@@@@@@@@@@@@@@@" ;

// ARIBR[hShift-JISR[hɕϊ
void CTsStream::Convert_jpn_arib_char(char *str,size_t len,char *ret_buf,size_t ret_len)
{
	unsigned char *p=(unsigned char*)str;

	size_t pos,ret_pos,write_size;	
	unsigned char c1,c2;

	unsigned char typG0 = CODE_JPN_KANJI;
	unsigned char typG1 = CODE_JPN_ALPHANUMERIC;
	unsigned char typG2 = CODE_JPN_HIRAGANA;
	unsigned char typG3 = CODE_JPN_KATAKANA;

	unsigned char *modGL  = &typG0;
	unsigned char *modGR  = &typG2;
	unsigned char *modGLs = NULL; // SingleShift
	unsigned char modeLR,mode;

	// convert 2byte code
	pos=ret_pos=0;
	for(pos=0;pos<len&&p[pos];)
	{
		char code_len=1;
		c1=c2=0;
		modeLR=CODE_JPN_MODE_NONE;
		mode=0;
		write_size=0;
		modGLs=NULL;
		unsigned char *sel_type=NULL;
		unsigned char new_type=0;
		if( p[pos] <= 0x20 )
		{
			// C0 Area
			switch( p[pos] )
			{
			case 0x0F: modGL = &typG0;	break;
			case 0x0E: modGL = &typG1;	break;
			case 0x1B:
				{
					// ESC
					code_len=2;
					switch(p[pos+1])
					{
					case 0x6E: modGL = &typG2; break;
					case 0x6F: modGL = &typG3; break;
					case 0x7E: modGR = &typG1; break;
					case 0x7D: modGR = &typG2; break;
					case 0x7C: modGR = &typG3; break;
					case 0x28:
						{
							sel_type = &typG0;
							if(p[pos+2]!=0x20)
							{
								new_type = p[pos+3]; code_len=4;
							} else {
								new_type = p[pos+2]; code_len=3;
							}									
						}
						break;
					case 0x29:
						{
							sel_type = &typG1;
							if(p[pos+2]==0x20)
							{
								new_type = p[pos+3]; code_len=4;
							} else {
								new_type = p[pos+2]; code_len=3;
							}
						}
						break;
					case 0x2A:
						{
							sel_type = &typG2;
							if(p[pos+2]==0x20)
							{
								new_type = p[pos+3]; code_len=4;
							} else {
								new_type = p[pos+2]; code_len=3;
							}
						}
						break;
					case 0x2B:
						{
							sel_type = &typG3;
							if(p[pos+2]==0x20)
							{
								new_type = p[pos+3]; code_len=4;
							} else {
								new_type = p[pos+2]; code_len=3;
							}							
						}
						break;
					case 0x24:
						{
							switch(p[pos+2])
							{
							case 0x28:
								{
									sel_type = &typG0;
									if(p[pos+3]==0x20)
									{
										new_type = p[pos+4]; code_len=5;
									} else {
										sel_type = NULL;
									}
								}
								break;
							case 0x29:
								{
									sel_type = &typG1;
									if(p[pos+3]==0x20)
									{
										new_type = p[pos+4]; code_len=5;
									} else {
										new_type = p[pos+3]; code_len=4;
									}
								}
								break;
							case 0x2A:
								{
									sel_type = &typG2;
									if(p[pos+3]==0x20)
									{
										new_type = p[pos+4]; code_len=5;
									} else {
										new_type = p[pos+3]; code_len=4;
									}
								}
								break;
							case 0x2B:
								{
									sel_type = &typG3;
									if(p[pos+3]==0x20)
									{
										new_type = p[pos+4]; code_len=5;
									} else {
										new_type = p[pos+3]; code_len=4;
									}
								}
								break;
							default:
								{
									sel_type = &typG0;
									new_type = p[pos+2]; code_len=3;
								}
							}
						}
						break;
					}					
					break;					
				}
				break;
			case 0x19: modGLs = &typG2; break;
			case 0x1D: modGLs = &typG3; break;
			}
			if(sel_type)
			{
				switch(new_type)
				{
				case 0x42: *sel_type=CODE_JPN_KANJI;            break;
				case 0x4A: *sel_type=CODE_JPN_ALPHANUMERIC;     break;
				case 0x30: *sel_type=CODE_JPN_HIRAGANA;         break;
				case 0x31: *sel_type=CODE_JPN_KATAKANA;         break;
				case 0x32: *sel_type=CODE_JPN_MOSAIC_A;         break;
				case 0x33: *sel_type=CODE_JPN_MOSAIC_B;         break;
				case 0x34: *sel_type=CODE_JPN_MOSAIC_C;         break;
				case 0x35: *sel_type=CODE_JPN_MOSAIC_D;         break;
				case 0x36: *sel_type=CODE_JPN_PRO_ALPHANUMERIC; break;			
				case 0x37: *sel_type=CODE_JPN_PRO_HIRAGANA;     break;
				case 0x38: *sel_type=CODE_JPN_PRO_KATAKANA;     break;
				case 0x49: *sel_type=CODE_JPN_X0201_KATAKANA;   break;
				case 0x39: *sel_type=CODE_JPN_JIS_KANJI_1;      break;				
				case 0x3A: *sel_type=CODE_JPN_JIS_KANJI_2;      break;
				case 0x3B: *sel_type=CODE_JPN_ADDITIONAL;       break;
				default: break;
				}
			}
			write_size=0;
		} else if( p[pos] >= 0x21 && p[pos] <= 0x7E ) {
			// GL Area
			modeLR = CODE_JPN_MODE_GL;
			mode=*modGL;
		} else if( p[pos] >= 0x7F && p[pos] <= 0xA0 ) {
			// C1 Area
			write_size=0;
		} else {
			// GR Area
			modeLR = CODE_JPN_MODE_GR;
			mode=*modGR;
		}
		if(modGLs||modeLR!=CODE_JPN_MODE_NONE)
		{
			unsigned char code_shift=0;
			switch(modeLR)
			{
			case CODE_JPN_MODE_GL:
				{
					code_shift=0x00;
				}
				break;
			case CODE_JPN_MODE_GR:
				{
					code_shift=0x80;
				}
				break;
			}
			if(modGLs)
			{
				code_shift=0x00;
				mode=*modGLs;
				pos++;
			}
			switch(mode)
			{
			case CODE_JPN_KANJI:
			case CODE_JPN_JIS_KANJI_1:
			case CODE_JPN_JIS_KANJI_2:

				{
					char t;
					t=p[pos+1];
					if(p[pos]&0x01)
					{
						c2=t+0x1F;			
					} else {
						c2=t+0x7D;
					}
					t=p[pos];
					if(t<0x5f)
					{
						c1=(t+1)/2+0x70;
					} else {
						c1=(t+1)/2+0xb0;
					}
					if(c2>0x7e) c2++;
					code_len=2;
					write_size=2;
				}
				break;
			case CODE_JPN_ALPHANUMERIC:
				{
					c1=jpn_argb_alp_num_table[(p[pos]-code_shift)*2];
					c2=jpn_argb_alp_num_table[(p[pos]-code_shift)*2+1];
					write_size=2;
				}
				break;
			case CODE_JPN_HIRAGANA:
				{
					c1=jpn_argb_hiragana_table[(p[pos]-code_shift)*2];
					c2=jpn_argb_hiragana_table[(p[pos]-code_shift)*2+1];
					write_size=2;
				}
				break;
			case CODE_JPN_KATAKANA:
				{
					c1=jpn_argb_katakana_table[(p[pos]-code_shift)*2];
					c2=jpn_argb_katakana_table[(p[pos]-code_shift)*2+1];
					write_size=2;
				}
				break;
			case CODE_JPN_ADDITIONAL:
				{
					c1=p[pos+0];
					c2=p[pos+1];
					code_len=2;
					write_size=0;
				}
				break;
			}
		}
		if(ret_buf&&ret_pos+write_size+1<ret_len)
		{
			switch(write_size)
			{
			case 1:
				{
					ret_buf[ret_pos]=c1;
					ret_pos+=2;
				}
				break;
			case 2:
				{
					ret_buf[ret_pos]=c1;
					ret_buf[ret_pos+1]=c2;
					ret_pos+=2;
				}
				break;
			}
		}
		pos+=code_len;
	}
	if(ret_buf)
	{
		if(ret_pos+1<ret_len)
		{
			ret_buf[ret_pos]='\0';
		} else {		
			ret_buf[ret_len-1]='\0';
		}
	}
}

// EXtɕϊ
void CTsStream::Convert_mjd_to_date(unsigned short mjd,long *p_year,long *p_month,long *p_day)
{
	double ajd = (double)mjd + 2400000.5;
	double t=ajd+0.5;
	long   z=(long)t;
	double f=t-z;
	long a;
	if(z<2299161)
	{
		a=z;
	} else {
		long x=(long)((z-1867216.25)/36524.25);
		a=z+1+x-(x/4); 
	}
	double b=a+1524;
	double c=(long)((b-122.1)/365.25);
	double d=(long)(365.25*c);
	double e=(long)((b-d)/30.6001);
	*p_day  =(long)(b-d-(long)(30.6001*e)+f);
	*p_month=(long)(e<14?e-1:e-13);
	*p_year =(long)(*p_month>2?c-4716:c-4715);
}

// tEXɕϊ({͏ł킷̂ȗĐ\)
WORD CTsStream::Convert_date_to_mjd(long year,long month,long day)
{
	if(month<=2)
	{
		year--;
		month+=12;
	}
	WORD mjd=(WORD)((int)(365.25 * year) + (int)(year / 400) - (int)(year / 100) + (int)(30.59 * (month - 2)) + day+0.5 - 678912);
	return mjd;
}

// abc^CR[hɕϊ
DWORD CTsStream::Convert_bcd_to_time(unsigned char bcd_hour,unsigned char bcd_min,unsigned char bcd_sec)
{
	int h=( (((bcd_hour&0xf0)>>4)*10)+(bcd_hour&0x0f));
	int m=( (((bcd_min&0xf0)>>4)*10)+(bcd_min&0x0f));
	int s=( (((bcd_sec&0xf0)>>4)*10)+(bcd_sec&0x0f));
	return h*3600+m*60+s;
}

// ^CR[h𕪉
void CTsStream::Convert_time(DWORD time,long *p_hour,long *p_min,long *p_sec)
{
	long h=time/60/60;
	long m=(time-h*60*60)/60;
	long s=(time-h*60*60-m*60);
	*p_hour=h;
	*p_min=m;
	*p_sec=s;
}

// abcDWORDɕϊ
DWORD CTsStream::Convert_bcd_to_dword(DWORD dwData)
{
	DWORD dwRet=0;
	for(int i=0,j=1;i<8;i++,j*=10)
	{
		dwRet += ((dwData>>(i*4))&0xF)*j;
	}
	return dwRet;
}